@@ -27,6 +27,8 @@
 #include "playback.h"
 #include "auddrct.h"
 #include "playlist.h"
+#include "ui_fileopener.h"
+#include "ui_jumptotrack.h"
 
 /* player */
 
@@ -39,15 +41,14 @@ drct_quit ( void )
 void
 drct_eject ( void )
 {
-    gboolean play_button = FALSE;
+    gboolean play_button = TRUE;
     hook_call("filebrowser show", &play_button);
 }
 
 void
 drct_jtf_show ( void )
 {
-    gboolean show = TRUE;
-    hook_call("ui jump to track show", &show);
+    ui_jump_to_track ();
 }
 
 gboolean
@@ -88,20 +89,26 @@ drct_pl_win_toggle ( gboolean show )
 
 void drct_activate(void)
 {
-    gtk_window_present(GTK_WINDOW(mainwin));
+    g_message("implement me");
 }
 
 /* playback */
 
+void
+drct_initiate ( void )
+{
+    playback_initiate();
+}
+
 void
 drct_play ( void )
 {
     if (playback_get_paused())
         playback_pause();
     else if (playlist_get_length(playlist_get_active()))
-        event_queue("playback initiate", (gpointer)0xdeadbeef); // to avoid crash at startup. --yaz
+        playback_initiate ();
     else
-        mainwin_eject_pushed();
+        run_filebrowser(TRUE);
     return;
 }
 
@@ -118,7 +125,6 @@ drct_stop ( void )
     ip_data.stop = TRUE;
     playback_stop();
     ip_data.stop = FALSE;
-    mainwin_clear_song_info();
     return;
 }
 
@@ -149,12 +155,16 @@ drct_get_info( gint *rate, gint *freq, gint *nch)
 gint
 drct_get_time ( void )
 {
-    gint time;
+    return playback_get_time ();
+}
+
+gint
+drct_get_length ( void )
+{
     if (playback_get_playing())
-        time = playback_get_time();
+        return playback_get_length();
     else
-        time = 0;
-    return time;
+        return -1;
 }
 
 void
@@ -279,7 +289,7 @@ drct_pl_repeat_is_enabled( void )
 void
 drct_pl_repeat_toggle( void )
 {
-    mainwin_repeat_pushed(!cfg.repeat);
+    g_message("implement me");
     return;
 }
 
@@ -292,7 +302,7 @@ drct_pl_repeat_is_shuffled( void )
 void
 drct_pl_shuffle_toggle( void )
 {
-    mainwin_shuffle_pushed(!cfg.shuffle);
+    g_message("implement me");
     return;
 }
 
@@ -336,7 +346,6 @@ void
 drct_pl_clear ( void )
 {
     playlist_clear(playlist_get_active());
-    mainwin_clear_song_info();
     return;
 }
 
@@ -345,9 +354,7 @@ drct_pl_clear ( void )
 void
 drct_pl_delete ( gint pos )
 {
-    GDK_THREADS_ENTER();
     playlist_delete_index(playlist_get_active(), pos);
-    GDK_THREADS_LEAVE();
 }
 
 void
@@ -373,9 +380,7 @@ drct_pl_ins_url_string( gchar * string, gint pos )
 void
 drct_pl_add_url_string( gchar * string )
 {
-    GDK_THREADS_ENTER();
     playlist_add_url(playlist_get_active(), string);
-    GDK_THREADS_LEAVE();
 }
 
 void
