@@ -23,6 +23,8 @@
  *  Audacious or using our public API to be a derived work.
  */
 
+#include <stdint.h>
+
 /* #define AUD_DEBUG */
 
 #ifdef HAVE_CONFIG_H
@@ -42,51 +44,26 @@
 #include "output.h"
 #include "playback.h"
 #include "pluginenum.h"
-#include "strings.h"
+#include "audstrings.h"
 #include "tuple.h"
 #include "tuple_formatter.h"
-#include "ui_main.h"
 #include "util.h"
 #include "visualization.h"
-#include "ui_skinned_playstatus.h"
 #include "hook.h"
 
 #include "vfs.h"
 #include "vfs_buffer.h"
 #include "vfs_buffered_file.h"
 
-#include "libSAD.h"
-
-#define FMT_FRACBITS(a) ( (a) == FMT_FIXED32 ? __AUDACIOUS_ASSUMED_MAD_F_FRACBITS__ : 0 )
-
-G_LOCK_DEFINE_STATIC(vis_mutex);
-
-struct _VisNode {
-    gint time;
-    gint nch;
-    gint length;                /* number of samples per channel */
-    gint16 data[2][512];
-};
-
-typedef struct _VisNode VisNode;
-
-
 InputPluginData ip_data = {
     NULL,
     NULL,
     FALSE,
     FALSE,
     FALSE,
-    FALSE,
     NULL
 };
 
-static GList *vis_list = NULL;
-static int volume_l = -1, volume_r = -1;
-static SAD_dither_t *sad_state = NULL;
-static gint sad_nch = -1;
-static AFormat sad_fmt = -1;
-
 InputPlayback *
 get_current_input_playback(void)
 {
@@ -137,87 +114,12 @@ input_stringify_disabled_list(void)
     return g_string_free(list, FALSE);
 }
 
-void
-free_vis_data(void)
-{
-    GList *iter;
-
-    G_LOCK(vis_mutex);
-
-    MOWGLI_ITER_FOREACH(iter, vis_list)
-        g_slice_free(VisNode, iter->data);
-
-    g_list_free(vis_list);
-    vis_list = NULL;
-
-    G_UNLOCK(vis_mutex);
-}
-
 InputVisType
 input_get_vis_type()
 {
     return INPUT_VIS_OFF;
 }
 
-static SAD_dither_t*
-init_sad(AFormat fmt, gint nch)
-{
-    gint ret;
-
-    SAD_buffer_format in, out;
-    in.sample_format = sadfmt_from_afmt(fmt);
-    in.fracbits = FMT_FRACBITS(fmt);
-    in.channels = nch;
-    in.channels_order = SAD_CHORDER_INTERLEAVED;
-    in.samplerate = 0;
-    
-    out.sample_format = SAD_SAMPLE_S16;
-    out.fracbits = 0;
-    out.channels = nch;
-    out.channels_order = SAD_CHORDER_SEPARATED; /* sic! --asphyx */
-    out.samplerate = 0;
-
-    AUDDBG("fmt=%d, nch=%d\n", fmt, nch);
-    SAD_dither_t *state = SAD_dither_init(&in, &out, &ret);
-    if (state != NULL) SAD_dither_set_dither(state, FALSE);
-    return state;
-}
-
-void
-input_add_vis_pcm(gint time, AFormat fmt, gint nch, gint length, gpointer ptr)
-{
-    VisNode *vis_node;
-    gint max;
-    
-    if (nch > 2) return;
-
-    if (sad_state == NULL || nch != sad_nch || fmt != sad_fmt) {
-        if(sad_state != NULL) SAD_dither_free(sad_state);
-        sad_state = init_sad(fmt, nch);
-        if(sad_state == NULL) return;
-        sad_nch = nch;
-        sad_fmt = fmt;
-    }
-
-    max = length / nch / FMT_SIZEOF(fmt);
-    max = CLAMP(max, 0, 512);
-
-    vis_node = g_slice_new0(VisNode);
-    vis_node->time = time;
-    vis_node->nch = nch;
-    vis_node->length = max;
-
-    gint16 *tmp[2];
-
-    tmp[0] = vis_node->data[0];
-    tmp[1] = vis_node->data[1];
-    SAD_dither_process_buffer(sad_state, ptr, tmp, max);
-
-    G_LOCK(vis_mutex);
-    vis_list = g_list_append(vis_list, vis_node);
-    G_UNLOCK(vis_mutex);
-}
-
 void
 input_dont_show_warning(GtkObject * object, gpointer user_data)
 {
@@ -390,8 +292,12 @@ input_check_file(const gchar *filename, gboolean loading)
     }
 
 
-    // open the file with vfs sub-system
-    fd = vfs_buffered_file_new_from_uri(filename_proxy);
+    /* open the file with vfs sub-system
+     * FIXME! XXX! buffered VFS file does not handle mixed seeks and reads/writes
+     * correctly! As a temporary workaround, switching to unbuffered ... -ccr
+     */
+    //fd = vfs_buffered_file_new_from_uri(filename_proxy);
+    fd = vfs_fopen(filename_proxy, "rb");
 
     if (!fd) {
         printf("Unable to read from %s, giving up.\n", filename_proxy);
@@ -402,18 +308,18 @@ input_check_file(const gchar *filename, gboolean loading)
 
     // apply mimetype check. note that stdio does not support mimetype check.
     mimetype = vfs_get_metadata(fd, "content-type");
-    if ((ip = mime_get_plugin(mimetype)) != NULL && ip->enabled) {
-        while(1) {
-            if (!ip || !ip->enabled)
-                continue;
+    if (mimetype) {
+        ip = mime_get_plugin(mimetype);
+        g_free(mimetype);
+    } else
+        ip = NULL;
 
-            pr = input_do_check_file(ip, fd, filename_proxy, loading);
-
-            if(pr) {
-                g_free(filename_proxy);
-                vfs_fclose(fd);
-                return pr;
-            }
+    if (ip && ip->enabled) {
+        pr = input_do_check_file(ip, fd, filename_proxy, loading);
+        if (pr) {
+            g_free(filename_proxy);
+            vfs_fclose(fd);
+            return pr;
         }
     }
 
@@ -421,8 +327,8 @@ input_check_file(const gchar *filename, gboolean loading)
     // apply ext_hash check
     if(cfg.use_extension_probing) {
         use_ext_filter =
-            (fd && (!g_strncasecmp(filename_proxy, "/", 1) ||
-                    !g_strncasecmp(filename_proxy, "file://", 7))) ? TRUE : FALSE;
+            (fd && (!g_ascii_strncasecmp(filename_proxy, "/", 1) ||
+                    !g_ascii_strncasecmp(filename_proxy, "file://", 7))) ? TRUE : FALSE;
     }
 
     if(use_ext_filter) {
@@ -434,7 +340,7 @@ input_check_file(const gchar *filename, gboolean loading)
         base = g_path_get_basename(realfn);
         g_free(realfn);
         ext = strrchr(base, '.');
-    
+
         if(ext) {
             lext = g_ascii_strdown(ext+1, -1);
             list_hdr = g_hash_table_lookup(ext_hash, lext);
@@ -488,26 +394,6 @@ input_check_file(const gchar *filename, gboolean loading)
     return NULL;
 }
 
-
-void
-input_set_eq(gint on, gfloat preamp, gfloat * bands)
-{
-    InputPlayback *playback;
-
-    if (!ip_data.playing)
-        return;
-
-    if ((playback = get_current_input_playback()) == NULL)
-        return;
-
-    if (playback->plugin->set_eq)
-    {
-        plugin_set_current((Plugin *)(playback->plugin));
-        playback->plugin->set_eq(on, preamp, bands);
-    }
-}
-
-
 Tuple *
 input_get_song_tuple(const gchar * filename)
 {
@@ -557,7 +443,6 @@ input_get_song_tuple(const gchar * filename)
         }
 
         tuple_associate_int(input, FIELD_LENGTH, NULL, -1);
-
     }
 
     g_free(filename_proxy);
@@ -633,8 +518,8 @@ input_general_file_info_box(const gchar * filename, InputPlugin * ip)
 
     cancel = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
     g_signal_connect_swapped(G_OBJECT(cancel), "clicked",
-                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                             GTK_OBJECT(window));
+                             G_CALLBACK(gtk_widget_destroy),
+                             window);
     GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);
 
@@ -711,72 +596,29 @@ input_scan_dir(const gchar * path)
 void
 input_get_volume(gint * l, gint * r)
 {
-    if (volume_l == -1 || volume_r == -1)
-	output_get_volume(&volume_l, &volume_r);
+    if (ip_data.playing && ip_data.current_input_playback &&
+     ip_data.current_input_playback->plugin->get_volume &&
+     ip_data.current_input_playback->plugin->get_volume (l, r))
+        return;
 
-    *l = volume_l;
-    *r = volume_r;
+    output_get_volume (l, r);
 }
 
 void
 input_set_volume(gint l, gint r)
 {
-    InputPlayback *playback;
-
     gint h_vol[2];
 
     h_vol[0] = l;
     h_vol[1] = r;
     hook_call("volume set", h_vol);
-    
-    if (playback_get_playing())
-        if ((playback = get_current_input_playback()) != NULL)
-	    if (playback->plugin->set_volume != NULL)
-	    {
-	        plugin_set_current((Plugin *)(playback->plugin));
-	        if (playback->plugin->set_volume(l, r))
-		    return;
-	    }
 
-    output_set_volume(l, r);
+    if (ip_data.playing && ip_data.current_input_playback &&
+     ip_data.current_input_playback->plugin->set_volume &&
+     ip_data.current_input_playback->plugin->set_volume (l, r))
+        return;
 
-    volume_l = l;
-    volume_r = r;
-}
-
-void
-input_update_vis(gint time)
-{
-    GList *node;
-    VisNode *vis = NULL, *visnext = NULL;
-    gboolean found = FALSE;
-
-    G_LOCK(vis_mutex);
-    node = vis_list;
-    while (g_list_next(node) && !found) {
-        visnext = g_list_next(node)->data;
-        vis = node->data;
-
-        if (vis->time >= time)
-            break;
-
-        vis_list = g_list_delete_link(vis_list, node);
-
-        if (visnext->time >= time) {
-            found = TRUE;
-            break;
-        }
-        g_free(vis);
-        node = vis_list;
-    }
-    G_UNLOCK(vis_mutex);
-
-    if (found) {
-        vis_send_data(vis->data, vis->nch, vis->length);
-        g_free(vis);
-    }
-    else
-        vis_send_data(NULL, 0, 0);
+    output_set_volume (l, r);
 }
 
 /* FIXME: move this somewhere else */
@@ -784,24 +626,5 @@ void
 input_set_info_text(gchar *text)
 {
     gchar *title = g_strdup(text);
-    event_queue("title change", title);
-}
-
-void
-input_set_status_buffering(gboolean status)
-{
-    if (!playback_get_playing())
-        return;
-
-    if (!get_current_input_playback())
-        return;
-
-    ip_data.buffering = status;
-
-    g_return_if_fail(mainwin_playstatus != NULL);
-
-    if (ip_data.buffering == TRUE && mainwin_playstatus != NULL && UI_SKINNED_PLAYSTATUS(mainwin_playstatus)->status == STATUS_STOP)
-        UI_SKINNED_PLAYSTATUS(mainwin_playstatus)->status = STATUS_PLAY;
-
-    ui_skinned_playstatus_set_buffering(mainwin_playstatus, ip_data.buffering);
+    event_queue_with_data_free("title change", title);
 }
