@@ -1,5 +1,5 @@
 /*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2008  Audacious team
+ *  Copyright (C) 2005-2009  Audacious team
  *
  *  Based on BMP:
  *  Copyright (C) 2003-2004  BMP development team.
@@ -25,10 +25,14 @@
 
 /*#define AUD_DEBUG*/
 
+#include <stdint.h>
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include "audconfig.h"
+#include "equalizer.h"
 #include "output.h"
 #include "main.h"
 #include "input.h"
@@ -42,8 +46,8 @@
 #include "pluginenum.h"
 
 #include "effect.h"
+#include "vis_runner.h"
 #include "volumecontrol.h"
-#include "visualization.h"
 
 #include "libSAD.h"
 #include "util.h"
@@ -51,11 +55,12 @@
 
 #include <math.h>
 
-#ifdef USE_SRC
+#ifdef USE_SAMPLERATE
 # include "src_flow.h"
 #endif
 
-#define FMT_FRACBITS(a) ( (a) == FMT_FIXED32 ? __AUDACIOUS_ASSUMED_MAD_F_FRACBITS__ : 0 )
+#define IS_S16_NE(a) ((a) == FMT_S16_NE || (G_BYTE_ORDER == G_LITTLE_ENDIAN && \
+ (a) == FMT_S16_LE) || (G_BYTE_ORDER == G_BIG_ENDIAN && (a) == FMT_S16_BE))
 
 OutputPluginData op_data = {
     NULL,
@@ -100,55 +105,27 @@ get_current_output_plugin(void)
 void
 set_current_output_plugin(gint i)
 {
-    gboolean playing;
-    OutputPlugin *op = get_current_output_plugin();
+    char playing;
+    Playlist * playlist;
+    int time, position;
 
-    GList *node = g_list_nth(op_data.output_list, i);
-    if (!node) {
-        op_data.current_output_plugin = NULL;
-        return;
+    playing = playback_get_playing ();
+
+    if (playing)
+    {
+        playlist = playlist_get_active ();
+        position = playlist_get_position (playlist);
+        time = playback_get_time ();
+        playback_stop();
     }
 
-    op_data.current_output_plugin = node->data;
+    op_data.current_output_plugin = g_list_nth(op_data.output_list, i)->data;
 
-    playing = playback_get_playing();
-
-    if (playing == TRUE)
+    if (playing)
     {
-        guint time, pos;
-        PlaylistEntry *entry;
-	
-	plugin_set_current((Plugin *)op);
-
-        /* don't stop yet, get the seek time and playlist position first */
-        pos = playlist_get_position(playlist_get_active());
-        time = op->output_time();
-
-        /* reset the audio system */
-        mainwin_stop_pushed();
-        op->close_audio();
-
-        g_usleep(300000);
-
-        /* wait for the playback thread to come online */
-        while (playback_get_playing())
-            g_message("waiting for audio system shutdown...");
-
-        /* wait for the playback thread to come online */
-        playlist_set_position(playlist_get_active(), pos);
-        entry = playlist_get_entry_to_play(playlist_get_active());
-        playback_play_file(entry);
-
-        while (!playback_get_playing())
-        {
-            gtk_main_iteration();
-                g_message("waiting for audio system startup...");
-        }
-
-        /* and signal a reseek */
-        if (playlist_get_current_length(playlist_get_active()) > -1 &&
-            time <= (playlist_get_current_length(playlist_get_active())))
-            playback_seek(time / 1000);
+        playlist_set_position (playlist, position);
+        playback_initiate ();
+        playback_seek (time / 1000);
     }
 }
 
@@ -183,39 +160,32 @@ output_configure(gint i)
 void
 output_get_volume(gint * l, gint * r)
 {
-    *l = *r = -1;
-
-    if (!op_data.current_output_plugin)
-        return;
-
-    if (!op_data.current_output_plugin->get_volume)
-        return;
-
     if (cfg.software_volume_control)
-        volumecontrol_get_volume_state(l, r);
+    {
+        * l = cfg.sw_volume_left;
+        * r = cfg.sw_volume_right;
+    }
+    else if (op_data.current_output_plugin &&
+     op_data.current_output_plugin->get_volume)
+        op_data.current_output_plugin->get_volume (l, r);
     else
     {
-        plugin_set_current((Plugin *)op_data.current_output_plugin);
-        op_data.current_output_plugin->get_volume(l, r);
+        * l = -1;
+        * r = -1;
     }
 }
 
 void
 output_set_volume(gint l, gint r)
 {
-    if (!op_data.current_output_plugin)
-        return;
-
-    if (!op_data.current_output_plugin->set_volume)
-        return;
-
     if (cfg.software_volume_control)
-        volumecontrol_set_volume_state(l, r);
-    else
     {
-        plugin_set_current((Plugin *)op_data.current_output_plugin);
-        op_data.current_output_plugin->set_volume(l, r);
+        cfg.sw_volume_left = l;
+        cfg.sw_volume_right = r;
     }
+    else if (op_data.current_output_plugin &&
+     op_data.current_output_plugin->set_volume)
+        op_data.current_output_plugin->set_volume (l, r);
 }
 
 void
@@ -249,10 +219,7 @@ get_output_time(void)
 
 static SAD_dither_t *sad_state_to_float = NULL;
 static SAD_dither_t *sad_state_from_float = NULL;
-static float *sad_float_buf = NULL;
-static void *sad_out_buf = NULL;
-static gsize sad_float_buf_length = 0;
-static gsize sad_out_buf_length = 0;
+
 static ReplayGainInfo replay_gain_info = {
     .track_gain = 0.0,
     .track_peak = 0.0,
@@ -289,6 +256,9 @@ reopen_audio(AFormat fmt, gint rate, gint nch)
     {
         plugin_set_current((Plugin *)op);
         op->close_audio();
+	op_state.fmt = 0;
+	op_state.rate = 0;
+	op_state.nch = 0;
     }
 
     plugin_set_current((Plugin *)op);
@@ -301,6 +271,8 @@ reopen_audio(AFormat fmt, gint rate, gint nch)
         op_state.rate = rate;
         op_state.nch = nch;
 
+        output_set_eq (cfg.equalizer_active, cfg.equalizer_preamp,
+         cfg.equalizer_bands);
         return TRUE;
     } else {
         return FALSE;
@@ -330,17 +302,31 @@ output_open_audio(AFormat fmt, gint rate, gint nch)
     if (bypass_dsp) {
         return TRUE;
     } else {
-#ifdef USE_SRC
+#ifdef USE_SAMPLERATE
         rate = src_flow_init(rate, nch); /* returns sample rate unchanged if resampling switched off */
 #endif
-    
+
         bit_depth = cfg.output_bit_depth;
-    
+
         AUDDBG("bit depth: %d\n", bit_depth);
-        output_fmt = (bit_depth == 24) ? FMT_S24_NE : FMT_S16_NE;
-        
+        switch (bit_depth) {
+            case 32:
+                output_fmt = FMT_S32_NE;
+                break;
+            case 24:
+                output_fmt = FMT_S24_NE;
+                break;
+            case 16:
+                output_fmt = FMT_S16_NE;
+                break;
+            default:
+                AUDDBG("unsupported bit depth requested %d\n", bit_depth);
+                output_fmt = FMT_S16_NE;
+                break;
+        }
+
         freeSAD();
-    
+
         AUDDBG("initializing dithering engine for 2 stage conversion: fmt%d --> float -->fmt%d\n", fmt, output_fmt);
         input_sad_fmt.sample_format = sadfmt_from_afmt(fmt);
         if (input_sad_fmt.sample_format < 0) return FALSE;
@@ -348,33 +334,33 @@ output_open_audio(AFormat fmt, gint rate, gint nch)
         input_sad_fmt.channels = nch;
         input_sad_fmt.channels_order = SAD_CHORDER_INTERLEAVED;
         input_sad_fmt.samplerate = 0;
-        
+
         output_sad_fmt.sample_format = SAD_SAMPLE_FLOAT;
         output_sad_fmt.fracbits = 0;
         output_sad_fmt.channels = nch;
         output_sad_fmt.channels_order = SAD_CHORDER_INTERLEAVED;
         output_sad_fmt.samplerate = 0;
-        
+
         sad_state_to_float = SAD_dither_init(&input_sad_fmt, &output_sad_fmt, &ret);
         if (sad_state_to_float == NULL) {
             AUDDBG("ditherer init failed (decoder's native --> float)\n");
             return FALSE;
         }
         SAD_dither_set_dither (sad_state_to_float, FALSE);
-        
+
         input_sad_fmt.sample_format = SAD_SAMPLE_FLOAT;
         input_sad_fmt.fracbits = 0;
         input_sad_fmt.channels = nch;
         input_sad_fmt.channels_order = SAD_CHORDER_INTERLEAVED;
         input_sad_fmt.samplerate = 0;
-        
+
         output_sad_fmt.sample_format = sadfmt_from_afmt(output_fmt);
         if (output_sad_fmt.sample_format < 0) return FALSE;
         output_sad_fmt.fracbits = FMT_FRACBITS(output_fmt);
         output_sad_fmt.channels = nch;
         output_sad_fmt.channels_order = SAD_CHORDER_INTERLEAVED;
         output_sad_fmt.samplerate = 0;
-        
+
         sad_state_from_float = SAD_dither_init(&input_sad_fmt, &output_sad_fmt, &ret);
         if (sad_state_from_float == NULL) {
             SAD_dither_free(sad_state_to_float);
@@ -382,9 +368,9 @@ output_open_audio(AFormat fmt, gint rate, gint nch)
             return FALSE;
         }
         SAD_dither_set_dither (sad_state_from_float, TRUE);
-        
+
         fmt = output_fmt;
-    
+
         if(replay_gain_info.album_peak == 0.0 && replay_gain_info.track_peak == 0.0) {
             AUDDBG("RG info isn't set yet. Filling replay_gain_info with default values.\n");
             replay_gain_info.track_gain = cfg.default_gain;
@@ -393,7 +379,7 @@ output_open_audio(AFormat fmt, gint rate, gint nch)
             replay_gain_info.album_peak = 0.01;
         }
         apply_replaygain_info(&replay_gain_info);
-    
+
         return reopen_audio(fmt, rate, nch);
     } /* bypass_dsp */
 }
@@ -416,23 +402,26 @@ output_close_audio(void)
 {
     OutputPlugin *op = get_current_output_plugin();
 
+    vis_runner_flush ();
     freeSAD();
-    
+
     AUDDBG("clearing RG settings\n");
     replay_gain_info.track_gain = 0.0;
     replay_gain_info.track_peak = 0.0;
     replay_gain_info.album_gain = 0.0;
     replay_gain_info.album_peak = 0.0;
 
-#ifdef USE_SRC
+#ifdef USE_SAMPLERATE
     src_flow_free();
 #endif
-    /* Do not close if there are still songs to play and the user has 
+    /* Do not close if there are still songs to play and the user has
      * not requested a stop.  --nenolod
      */
     Playlist *pl = playlist_get_active();
     if (ip_data.stop == FALSE &&
-       (playlist_get_position_nolock(pl) < playlist_get_length(pl) - 1)) {
+       (playlist_get_position_nolock(pl) < playlist_get_length(pl) - 1) &&
+       !cfg.stopaftersong &&
+       !(cfg.no_playlist_advance && !cfg.repeat)) {
             AUDDBG("leaving audio opened\n");
             return;
         }
@@ -455,6 +444,8 @@ output_flush(gint time)
 {
     OutputPlugin *op = get_current_output_plugin();
 
+    vis_runner_flush ();
+
     if (op == NULL)
         return;
 
@@ -508,81 +499,88 @@ output_pass_audio(InputPlayback *playback,
               int *going         /* 0 when time to stop  */
               )
 {
-    static Flow *visualization_flow = NULL;
     static Flow *postproc_flow = NULL;
     static Flow *legacy_flow = NULL;
     OutputPlugin *op = playback->output;
     gint writeoffs;
-    gpointer float_ptr;
-        
-    if (visualization_flow == NULL)
+    void * orig_ptr = ptr;
+    int orig_length = length;
+    void * old_ptr, * new_ptr;
+    int time = playback->output->written_time ();
+
+    if (fmt != FMT_FLOAT)
     {
-        visualization_flow = flow_new();
-        flow_link_element(visualization_flow, vis_flow);
+        int samples = length / FMT_SIZEOF (fmt);
+
+        length = sizeof (float) * samples;
+        new_ptr = malloc (length);
+
+        if (IS_S16_NE (fmt))
+            s16_to_float (ptr, new_ptr, samples);
+        else
+            SAD_dither_process_buffer (sad_state_to_float, ptr, new_ptr,
+             samples / nch);
+
+        if (ptr != orig_ptr) free (ptr);
+        ptr = new_ptr;
     }
-        
-    plugin_set_current((Plugin *)(playback->output));
-    gint time = playback->output->written_time();
-        
-    flow_execute(visualization_flow, time, &ptr, length, fmt, decoder_srate, nch);
 
-    if (!bypass_dsp) {
+    vis_runner_pass_audio (time, ptr, length / sizeof (float), nch);
 
-        if(length <= 0 || sad_state_from_float == NULL || sad_state_to_float == NULL) return;
-        
-        if (legacy_flow == NULL)
+    if (bypass_dsp)
+    {
+        if (ptr != orig_ptr) free (ptr);
+        ptr = orig_ptr;
+        length = orig_length;
+    }
+    else
+    {
+        if (! postproc_flow)
         {
-            legacy_flow = flow_new();
-            flow_link_element(legacy_flow, effect_flow);
-        }
-        
-        if (postproc_flow == NULL)
-        {
-            postproc_flow = flow_new();
-#ifdef USE_SRC
-            flow_link_element(postproc_flow, src_flow);
+            postproc_flow = flow_new ();
+#ifdef USE_SAMPLERATE
+            flow_link_element (postproc_flow, src_flow);
 #endif
-            flow_link_element(postproc_flow, equalizer_flow);
-            flow_link_element(postproc_flow, volumecontrol_flow);
+            flow_link_element (postproc_flow, equalizer_flow);
+            flow_link_element (postproc_flow, volumecontrol_flow);
         }
-    
-        int frames = length / nch / FMT_SIZEOF(fmt);
-        int len = frames * nch * sizeof(float);
-        if(sad_float_buf == NULL || sad_float_buf_length < len) {
-            sad_float_buf_length = len;
-            sad_float_buf = smart_realloc(sad_float_buf, &sad_float_buf_length);
+
+        old_ptr = ptr;
+        length = flow_execute (postproc_flow, time, & ptr, length,
+         FMT_FLOAT, decoder_srate, nch);
+        if (ptr != old_ptr && old_ptr != orig_ptr) free (old_ptr);
+
+        if (op_state.fmt != FMT_FLOAT)
+        {
+            int samples = length / sizeof (float);
+
+            length = FMT_SIZEOF (op_state.fmt) * samples;
+            new_ptr = malloc (length);
+
+            if (cfg.no_dithering && IS_S16_NE (op_state.fmt))
+                float_to_s16 (ptr, new_ptr, samples);
+            else
+                SAD_dither_process_buffer (sad_state_from_float, ptr, new_ptr,
+                 samples / nch);
+
+            if (ptr != orig_ptr) free (ptr);
+            ptr = new_ptr;
         }
-    
-        SAD_dither_process_buffer(sad_state_to_float, ptr, sad_float_buf, frames);
-        float_ptr = sad_float_buf;
-        
-        length = flow_execute(postproc_flow,
-                              time,
-                              &float_ptr,
-                              len,
-                              FMT_FLOAT,
-                              decoder_srate,
-                              nch);
-        
-        frames = length / nch / sizeof(float);
-        len = frames * nch * FMT_SIZEOF(op_state.fmt);
-        if(sad_out_buf == NULL || sad_out_buf_length < len) {
-            sad_out_buf_length = len;
-            sad_out_buf = smart_realloc(sad_out_buf, &sad_out_buf_length);
+
+        if (IS_S16_NE (op_state.fmt))
+        {
+            if (! legacy_flow)
+            {
+                legacy_flow = flow_new ();
+                flow_link_element (legacy_flow, effect_flow);
+            }
+
+            old_ptr = ptr;
+            length = flow_execute (legacy_flow, time, & ptr, length,
+             op_state.fmt, op_state.rate, op_state.nch);
+            if (ptr != old_ptr && old_ptr != orig_ptr) free (old_ptr);
         }
-    
-        SAD_dither_process_buffer(sad_state_from_float, float_ptr, sad_out_buf, frames);
-    
-        length = len;
-        ptr = sad_out_buf;
-    
-        if (op_state.fmt == FMT_S16_NE || (op_state.fmt == FMT_S16_LE && G_BYTE_ORDER == G_LITTLE_ENDIAN) ||
-                                          (op_state.fmt == FMT_S16_BE && G_BYTE_ORDER == G_BIG_ENDIAN)) {
-            length = flow_execute(legacy_flow, time, &ptr, length, op_state.fmt, op_state.rate, op_state.nch);
-        } else {
-            AUDDBG("legacy_flow can deal only with S16_NE streams\n"); /*FIXME*/
-        }
-    } /* !bypass_dsp */
+    }
 
     /**** write it out ****/
 
@@ -628,6 +626,8 @@ output_pass_audio(InputPlayback *playback,
 
         writeoffs += writable;
     }
+
+    if (ptr != orig_ptr) free (ptr);
 }
 
 /* called by input plugin when RG info available --asphyx */
@@ -650,13 +650,13 @@ apply_replaygain_info (ReplayGainInfo *rg_info)
         AUDDBG("SAD not initialized!\n");
         return;
     }
-    
+
     rg_enabled = cfg.enable_replay_gain;
     album_mode = cfg.replay_gain_album;
     mode.clipping_prevention = cfg.enable_clipping_prevention;
     mode.hard_limit = FALSE;
     mode.adaptive_scaler = cfg.enable_adaptive_scaler;
-    
+
     if(!rg_enabled) return;
 
     mode.mode = album_mode ? SAD_RG_ALBUM : SAD_RG_TRACK;
@@ -678,6 +678,19 @@ apply_replaygain_info (ReplayGainInfo *rg_info)
     AUDDBG("* track peak:          %f\n",     info.track_peak);
     AUDDBG("* album gain:          %+f dB\n", info.album_gain);
     AUDDBG("* album peak:          %f\n",     info.album_peak);
-    
+
     SAD_dither_apply_replaygain(sad_state_from_float, &info, &mode);
 }
+
+void output_plugin_cleanup(void)
+{
+  OutputPlugin *op = get_current_output_plugin();
+   op->init();
+   output_close_audio();
+   printf("output plugin cleanupn\n");
+}
+void output_plugin_reinit(void)
+{
+
+    printf("output plugin reinit \n");
+}
