@@ -32,36 +32,36 @@
 #endif
 
 #include <glib.h>
-#include <gmodule.h>
+#include <glib/gi18n.h>
 #include <glib/gprintf.h>
+#include <gmodule.h>
 #include <string.h>
 
-#include "main.h"
-#include "dock.h"
-#include "ui_main.h"
-#include "playback.h"
-#include "playlist.h"
-#include "strings.h"
-#include "util.h"
-
-#include "effect.h"
-#include "general.h"
-#include "input.h"
-#include "output.h"
-#include "visualization.h"
-#include "discovery.h"
-
 #include "pluginenum.h"
 
+#include "discovery.h"
+#include "effect.h"
+#include "general.h"
+#include "input.h"
+#include "main.h"
+#include "chardet.h"
+#include "output.h"
+#include "playback.h"
+#include "playlist.h"
+#include "audstrings.h"
+#include "util.h"
+#include "visualization.h"
+#include "preferences.h"
 #include "vfs_buffer.h"
 #include "vfs_buffered_file.h"
-
-#include "ui_preferences.h"
-#include "ui_fileinfopopup.h"
-
-#include "effect.h"
-#include "iir.h"
 #include "volumecontrol.h"
+#include "equalizer_preset.h"
+
+#include "ui_fileinfo.h"
+#include "ui_fileinfopopup.h"
+#include "ui_plugin_menu.h"
+#include "ui_preferences.h"
+
 
 const gchar *plugin_dir_list[] = {
     PLUGINSUBS,
@@ -70,6 +70,9 @@ const gchar *plugin_dir_list[] = {
 
 GHashTable *ext_hash = NULL;
 
+static void set_pvt_data(Plugin * plugin, gpointer data);
+static gpointer get_pvt_data(void);
+
 /*****************************************************************/
 
 static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
@@ -102,6 +105,13 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .vfs_buffered_file_new_from_uri = vfs_buffered_file_new_from_uri,
     .vfs_buffered_file_release_live_fd = vfs_buffered_file_release_live_fd,
 
+    .vfs_fget_le16 = vfs_fget_le16,
+    .vfs_fget_le32 = vfs_fget_le32,
+    .vfs_fget_le64 = vfs_fget_le64,
+    .vfs_fget_be16 = vfs_fget_be16,
+    .vfs_fget_be32 = vfs_fget_be32,
+    .vfs_fget_be64 = vfs_fget_be64,
+
     .cfg_db_open = cfg_db_open,
     .cfg_db_close = cfg_db_close,
 
@@ -145,6 +155,9 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .util_info_dialog = util_info_dialog,
     .get_gentitle_format = get_gentitle_format,
 
+    .smart_realloc = smart_realloc,
+    .sadfmt_from_afmt = sadfmt_from_afmt,
+
     .escape_shell_chars = escape_shell_chars,
     .str_append = str_append,
     .str_replace = str_replace,
@@ -153,11 +166,11 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .str_has_suffix_nocase = str_has_suffix_nocase,
     .str_has_suffixes_nocase = str_has_suffixes_nocase,
     .str_to_utf8_fallback = str_to_utf8_fallback,
-    .str_to_utf8 = str_to_utf8,
+    .str_to_utf8 = cd_str_to_utf8,
     .filename_to_utf8 = filename_to_utf8,
     .str_skip_chars = str_skip_chars,
     .convert_title_text = convert_title_text,
-    .chardet_to_utf8 = chardet_to_utf8,
+    .chardet_to_utf8 = cd_chardet_to_utf8,
 
     .playlist_container_register = playlist_container_register,
     .playlist_container_unregister = playlist_container_unregister,
@@ -186,7 +199,6 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .playlist_add_url = playlist_add_url,
     .playlist_ins_url = playlist_ins_url,
 
-    .playlist_check_pos_current = playlist_check_pos_current,
     .playlist_next = playlist_next,
     .playlist_prev = playlist_prev,
 
@@ -220,11 +232,10 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .playlist_remove_duplicates = playlist_remove_duplicates,
     .playlist_remove_dead_files = playlist_remove_dead_files,
 
-    .playlist_fileinfo_current = playlist_fileinfo_current,
-    .playlist_fileinfo = playlist_fileinfo,
+    .playlist_fileinfo_current = ui_fileinfo_show_current,
+    .playlist_fileinfo = ui_fileinfo_show,
 
     .playlist_delete_index = playlist_delete_index,
-    .playlist_delete_filenames = playlist_delete_filenames,
 
     .playlist_get_entry_to_play = playlist_get_entry_to_play,
 
@@ -283,13 +294,6 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .read_ini_string = read_ini_string,
     .read_ini_array = read_ini_array,
 
-    .xmms_convert_buffers_new = xmms_convert_buffers_new,
-    .xmms_convert_buffers_free = xmms_convert_buffers_free,
-    .xmms_convert_buffers_destroy = xmms_convert_buffers_destroy,
-    .xmms_convert_get_func = xmms_convert_get_func,
-    .xmms_convert_get_channel_func = xmms_convert_get_channel_func,
-    .xmms_convert_get_frequency_func = xmms_convert_get_frequency_func,
-
     .menu_plugin_item_add = menu_plugin_item_add,
     .menu_plugin_item_remove = menu_plugin_item_remove,
 
@@ -302,9 +306,9 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_eq_win_toggle = drct_eq_win_toggle,
     .drct_pl_win_is_visible = drct_pl_win_is_visible,
     .drct_pl_win_toggle = drct_pl_win_toggle,
-    .drct_set_skin = drct_set_skin,
     .drct_activate = drct_activate,
 
+    .drct_initiate = drct_initiate,
     .drct_play = drct_play,
     .drct_pause = drct_pause,
     .drct_stop = drct_stop,
@@ -313,6 +317,7 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_get_stopped = drct_get_stopped,
     .drct_get_info = drct_get_info,
     .drct_get_time = drct_get_time,
+    .drct_get_length = drct_get_length,
     .drct_seek = drct_seek,
     .drct_get_volume = drct_get_volume,
     .drct_set_volume = drct_set_volume,
@@ -348,12 +353,6 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .drct_pq_get_position = drct_pq_get_position,
     .drct_pq_get_queue_position = drct_pq_get_queue_position,
 
-    .formatter_new = formatter_new,
-    .formatter_destroy = formatter_destroy,
-    .formatter_associate = formatter_associate,
-    .formatter_dissociate = formatter_dissociate,
-    .formatter_format = formatter_format,
-
     .prefswin_page_new = prefswin_page_new,
     .prefswin_page_destroy = prefswin_page_destroy,
 
@@ -372,20 +371,38 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
     .flow_link_element = flow_link_element,
     .flow_unlink_element = flow_unlink_element,
     .effect_flow = effect_flow,
-    .iir_flow = iir_flow,
     .volumecontrol_flow = volumecontrol_flow,
 
-    .util_menu_main_show = util_menu_main_show,
-    
-    .get_dock_window_list = get_dock_window_list,
-    .dock_add_window = dock_add_window,
-    .dock_remove_window = dock_remove_window,
-    .dock_move_press = dock_move_press,
-    .dock_move_motion = dock_move_motion,
-    .dock_move_release = dock_move_release,
-    .dock_is_moving = dock_is_moving,
-
     .get_output_list = get_output_list,
+
+    .input_get_volume = input_get_volume,
+    .construct_uri = construct_uri,
+    .uri_to_display_basename = uri_to_display_basename,
+    .uri_to_display_dirname = uri_to_display_dirname,
+
+    .get_pvt_data = get_pvt_data,
+    .set_pvt_data = set_pvt_data,
+
+    .event_queue = event_queue,
+
+    .calc_mono_freq = calc_mono_freq,
+    .calc_mono_pcm = calc_mono_pcm,
+    .calc_stereo_pcm = calc_stereo_pcm,
+
+    .create_widgets = create_widgets,
+
+    .equalizer_read_presets = equalizer_read_presets,
+    .equalizer_write_preset_file = equalizer_write_preset_file,
+    .import_winamp_eqf = import_winamp_eqf,
+    .save_preset_file = save_preset_file,
+    .equalizer_read_aud_preset = equalizer_read_aud_preset,
+    .load_preset_file = load_preset_file,
+    .output_plugin_cleanup = output_plugin_cleanup,
+    .output_plugin_reinit = output_plugin_reinit,
+
+    .get_plugin_menu = get_plugin_menu,
+    .playlist_shift = playlist_shift,
+    .playlist_rescan = playlist_rescan,
 };
 
 /*****************************************************************/
@@ -393,7 +410,40 @@ static struct _AudaciousFuncTableV1 _aud_papi_v1 = {
 GList *lowlevel_list = NULL;
 extern GList *vfs_transports;
 
-mowgli_dictionary_t *plugin_dict = NULL;
+static mowgli_dictionary_t *plugin_dict = NULL;
+
+static GStaticPrivate cur_plugin_key = G_STATIC_PRIVATE_INIT;
+static mowgli_dictionary_t *pvt_data_dict = NULL;
+
+static mowgli_list_t *headers_list = NULL;
+
+void plugin_set_current(Plugin *plugin)
+{
+    g_static_private_set(&cur_plugin_key, plugin, NULL);
+}
+
+static Plugin *plugin_get_current(void)
+{
+    return g_static_private_get(&cur_plugin_key);
+}
+
+static void set_pvt_data(Plugin * plugin, gpointer data)
+{
+    mowgli_dictionary_elem_t *elem;
+
+    elem = mowgli_dictionary_find(pvt_data_dict, g_path_get_basename(plugin->filename));
+    if (elem == NULL)
+        mowgli_dictionary_add(pvt_data_dict, g_path_get_basename(plugin->filename), data);
+    else
+        elem->data = data;
+}
+
+static gpointer get_pvt_data(void)
+{
+    Plugin *cur_p = plugin_get_current();
+
+    return mowgli_dictionary_retrieve(pvt_data_dict, g_path_get_basename(cur_p->filename));
+}
 
 static gint
 inputlist_compare_func(gconstpointer a, gconstpointer b)
@@ -409,10 +459,7 @@ static gint
 outputlist_compare_func(gconstpointer a, gconstpointer b)
 {
     const OutputPlugin *ap = a, *bp = b;
-    if(ap->description && bp->description)
-        return strcasecmp(ap->description, bp->description);
-    else
-        return 0;
+    return (bp->probe_priority - ap->probe_priority);
 }
 
 static gint
@@ -499,28 +546,32 @@ plugin_get_plugin(const gchar *filename)
     return mowgli_dictionary_retrieve(plugin_dict, filename);
 }
 
+static void add_vis_pcm_dummy (gint time, AFormat format, gint channels, gint
+ length, void * data)
+{
+}
+
 static void
 input_plugin_init(Plugin * plugin)
 {
     InputPlugin *p = INPUT_PLUGIN(plugin);
 
     p->get_vis_type = input_get_vis_type;
-    p->add_vis_pcm = input_add_vis_pcm;
-    
+    p->add_vis_pcm = add_vis_pcm_dummy;
+
     /* Pretty const casts courtesy of XMMS's plugin.h legacy. Anyone
        else thinks we could use a CONST macro to solve the warnings?
        - descender */
     p->set_info = (void (*)(gchar *, gint, gint, gint, gint)) playlist_set_info_old_abi;
     p->set_info_text = input_set_info_text;
-    p->set_status_buffering = input_set_status_buffering;
 
     ip_data.input_list = g_list_append(ip_data.input_list, p);
-    
+
     p->enabled = TRUE;
 
     /* XXX: we need something better than p->filename if plugins
        will eventually provide multiple plugins --nenolod */
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 
     /* build the extension hash table */
     gint i;
@@ -544,9 +595,9 @@ static void
 output_plugin_init(Plugin * plugin)
 {
     OutputPlugin *p = OUTPUT_PLUGIN(plugin);
-    op_data.output_list = g_list_append(op_data.output_list, p);    
+    op_data.output_list = g_list_append(op_data.output_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 }
 
 static void
@@ -555,7 +606,7 @@ effect_plugin_init(Plugin * plugin)
     EffectPlugin *p = EFFECT_PLUGIN(plugin);
     ep_data.effect_list = g_list_append(ep_data.effect_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 }
 
 static void
@@ -564,7 +615,7 @@ general_plugin_init(Plugin * plugin)
     GeneralPlugin *p = GENERAL_PLUGIN(plugin);
     gp_data.general_list = g_list_append(gp_data.general_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 }
 
 static void
@@ -574,7 +625,7 @@ vis_plugin_init(Plugin * plugin)
     p->disable_plugin = vis_disable_plugin;
     vp_data.vis_list = g_list_append(vp_data.vis_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 }
 
 static void
@@ -583,7 +634,7 @@ discovery_plugin_init(Plugin * plugin)
     DiscoveryPlugin *p = DISCOVERY_PLUGIN(plugin);
     dp_data.discovery_list = g_list_append(dp_data.discovery_list, p);
 
-    mowgli_dictionary_add(plugin_dict, g_basename(p->filename), p);
+    mowgli_dictionary_add(plugin_dict, g_path_get_basename(p->filename), p);
 }
 
 /*******************************************************************/
@@ -591,26 +642,24 @@ discovery_plugin_init(Plugin * plugin)
 static void
 plugin2_dispose(GModule *module, const gchar *str, ...)
 {
-    gchar buf[4096];
+    gchar *buf;
     va_list va;
 
     va_start(va, str);
-    vsnprintf(buf, 4096, str, va);
+    buf = g_strdup_vprintf(str, va);
     va_end(va);
 
     g_message("*** %s\n", buf);
+    g_free(buf);
+
     g_module_close(module);
 }
 
 void
 plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
 {
-    InputPlugin **ip_iter;
-    OutputPlugin **op_iter;
-    EffectPlugin **ep_iter;
-    GeneralPlugin **gp_iter;
-    VisPlugin **vp_iter;
-    DiscoveryPlugin **dp_iter;
+    gint i, n;
+    mowgli_node_t *hlist_node;
 
     if (header->magic != PLUGIN_MAGIC)
         return plugin2_dispose(module, "plugin <%s> discarded, invalid module magic", filename);
@@ -619,6 +668,9 @@ plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
         return plugin2_dispose(module, "plugin <%s> discarded, wanting API version %d, we implement API version %d",
                                filename, header->api_version, __AUDACIOUS_PLUGIN_API__);
 
+    hlist_node = mowgli_node_create();
+    mowgli_node_add(header, hlist_node, headers_list);
+
     if (header->init)
         header->init();
 
@@ -626,68 +678,77 @@ plugin2_process(PluginHeader *header, GModule *module, const gchar *filename)
     header->priv_assoc->handle = module;
     header->priv_assoc->filename = g_strdup(filename);
 
+    n = 0;
+
     if (header->ip_list)
     {
-        for (ip_iter = header->ip_list; *ip_iter != NULL; ip_iter++)
+        for (i = 0; (header->ip_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*ip_iter)->filename = g_strdup(filename);
-            input_plugin_init(PLUGIN(*ip_iter));
+            PLUGIN((header->ip_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            input_plugin_init(PLUGIN((header->ip_list)[i]));
         }
     }
 
     if (header->op_list)
     {
-        for (op_iter = header->op_list; *op_iter != NULL; op_iter++)
+        for (i = 0; (header->op_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*op_iter)->filename = g_strdup(filename);
-            output_plugin_init(PLUGIN(*op_iter));
+            PLUGIN((header->op_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            output_plugin_init(PLUGIN((header->op_list)[i]));
         }
     }
 
     if (header->ep_list)
     {
-        for (ep_iter = header->ep_list; *ep_iter != NULL; ep_iter++)
+        for (i = 0; (header->ep_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*ep_iter)->filename = g_strdup(filename);
-            effect_plugin_init(PLUGIN(*ep_iter));
+            PLUGIN((header->ep_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            effect_plugin_init(PLUGIN((header->ep_list)[i]));
         }
     }
 
+
     if (header->gp_list)
     {
-        for (gp_iter = header->gp_list; *gp_iter != NULL; gp_iter++)
+        for (i = 0; (header->gp_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*gp_iter)->filename = g_strdup(filename);
-            general_plugin_init(PLUGIN(*gp_iter));
+            PLUGIN((header->gp_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            general_plugin_init(PLUGIN((header->gp_list)[i]));
         }
     }
 
     if (header->vp_list)
     {
-        for (vp_iter = header->vp_list; *vp_iter != NULL; vp_iter++)
+        for (i = 0; (header->vp_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*vp_iter)->filename = g_strdup(filename);
-            vis_plugin_init(PLUGIN(*vp_iter));
+            PLUGIN((header->vp_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            vis_plugin_init(PLUGIN((header->vp_list)[i]));
         }
     }
 
     if (header->dp_list)
     {
-        for (dp_iter = header->dp_list; *dp_iter != NULL; dp_iter++)
+        for (i = 0; (header->dp_list)[i] != NULL; i++, n++)
         {
-            PLUGIN(*dp_iter)->filename = g_strdup(filename);
-            discovery_plugin_init(PLUGIN(*dp_iter));
+            PLUGIN((header->dp_list)[i])->filename = g_strdup_printf("%s (#%d)", filename, n);
+            discovery_plugin_init(PLUGIN((header->dp_list)[i]));
         }
     }
+
+    if (header->interface)
+        interface_register(header->interface);
 }
 
 void
-plugin2_unload(PluginHeader *header)
+plugin2_unload(PluginHeader *header, mowgli_node_t *hlist_node)
 {
     GModule *module;
 
     g_return_if_fail(header->priv_assoc != NULL);
 
+    if (header->interface)
+        interface_deregister(header->interface);
+
     module = header->priv_assoc->handle;
 
     g_free(header->priv_assoc->filename);
@@ -696,6 +757,9 @@ plugin2_unload(PluginHeader *header)
     if (header->fini)
         header->fini();
 
+    mowgli_node_delete(hlist_node, headers_list);
+    mowgli_node_free(hlist_node);
+
     g_module_close(module);
 }
 
@@ -713,7 +777,7 @@ add_plugin(const gchar * filename)
     g_message("Loaded plugin (%s)", filename);
 
     if (!(module = g_module_open(filename, G_MODULE_BIND_LOCAL))) {
-        printf("Failed to load plugin (%s): %s\n", 
+        printf("Failed to load plugin (%s): %s\n",
                   filename, g_module_error());
         return;
     }
@@ -764,20 +828,32 @@ plugin_system_init(void)
     InputPlugin *ip;
     LowlevelPlugin *lp;
     DiscoveryPlugin *dp;
+    GtkWidget *dialog;
     gint dirsel = 0, i = 0;
+    gint prio;
 
     if (!g_module_supported()) {
-        report_error("Module loading not supported! Plugins will not be loaded.\n");
+        dialog =
+            gtk_message_dialog_new (NULL,
+                                    GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_MESSAGE_ERROR,
+                                    GTK_BUTTONS_CLOSE,
+                                    _("Module loading not supported! Plugins will not be loaded.\n"));
+        gtk_dialog_run (GTK_DIALOG (dialog));
+        gtk_widget_destroy (dialog);
         return;
     }
 
     plugin_dict = mowgli_dictionary_create(g_ascii_strcasecmp);
+    pvt_data_dict = mowgli_dictionary_create(g_ascii_strcasecmp);
+
+    headers_list = mowgli_list_create();
 
     /* make extension hash */
     ext_hash = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 
 #ifndef DISABLE_USER_PLUGIN_DIR
-    scan_plugins(bmp_paths[BMP_PATH_USER_PLUGIN_DIR]);
+    scan_plugins(aud_paths[BMP_PATH_USER_PLUGIN_DIR]);
     /*
      * This is in a separate lo
      * DiscoveryPlugin *dpop so if the user puts them in the
@@ -785,7 +861,7 @@ plugin_system_init(void)
      * first)                                                - Zinx
      */
     while (plugin_dir_list[dirsel]) {
-        dir = g_build_filename(bmp_paths[BMP_PATH_USER_PLUGIN_DIR],
+        dir = g_build_filename(aud_paths[BMP_PATH_USER_PLUGIN_DIR],
                                plugin_dir_list[dirsel++], NULL);
         scan_plugins(dir);
         g_free(dir);
@@ -837,37 +913,85 @@ plugin_system_init(void)
     g_free(cfg.enabled_dplugins);
     cfg.enabled_dplugins = NULL;
 
+    if (!cfg.outputplugin) {
+        for (prio = 10; prio >= 0; prio--) {
+            for (node = op_data.output_list; node; node = g_list_next(node)) {
+                op = OUTPUT_PLUGIN(node->data);
 
-    for (node = op_data.output_list; node; node = g_list_next(node)) {
-        op = OUTPUT_PLUGIN(node->data);
-        /*
-         * Only test basename to avoid problems when changing
-         * prefix.  We will only see one plugin with the same
-         * basename, so this is usually what the user want.
-         */
-        if (cfg.outputplugin && !strcmp(g_basename(cfg.outputplugin), g_basename(op->filename)))
-            op_data.current_output_plugin = op;
-        if (op->init)
-            op->init();
+                if (op->probe_priority != prio)
+                    continue;
+
+		if (op->init)
+		{
+		    OutputPluginInitStatus ret = op->init();
+		    if (ret == OUTPUT_PLUGIN_INIT_NO_DEVICES)
+		    {
+		        g_message("Plugin %s reports no devices. Attempting to avert disaster, trying others.\n",
+                                  g_path_get_basename(op->filename));
+                    }
+                    else if (ret == OUTPUT_PLUGIN_INIT_FAIL)
+                    {
+		        g_message("Plugin %s was unable to initialise. Attemping to avert disaster, trying others.\n",
+		                  g_path_get_basename(op->filename));
+                    }
+                    else if (ret == OUTPUT_PLUGIN_INIT_FOUND_DEVICES)
+                    {
+                        if (!op_data.current_output_plugin)
+                            op_data.current_output_plugin = op;
+                    }
+                    else if (!op_data.current_output_plugin)
+                    {
+		        g_message("Plugin %s did not report status, and no plugin has worked yet. Do you still need to convert it? Selecting for now...\n",
+		                  g_path_get_basename(op->filename));
+
+                        if (!op_data.current_output_plugin)
+                            op_data.current_output_plugin = op;
+                    }
+		}
+            }
+        }
+    }
+    else
+    {
+        for (node = op_data.output_list; node; node = g_list_next(node)) {
+            op = OUTPUT_PLUGIN(node->data);
+
+            if (op->init) {
+                plugin_set_current((Plugin *)op);
+                op->init();
+            }
+
+            if (!g_ascii_strcasecmp(g_path_get_basename(cfg.outputplugin), g_path_get_basename(op->filename)))
+                op_data.current_output_plugin = op;
+        }
     }
 
     for (node = ip_data.input_list; node; node = g_list_next(node)) {
         ip = INPUT_PLUGIN(node->data);
         if (ip->init)
+	{
+	    plugin_set_current((Plugin *)ip);
             ip->init();
+	}
     }
 
     for (node = dp_data.discovery_list; node; node = g_list_next(node)) {
         dp = DISCOVERY_PLUGIN(node->data);
         if (dp->init)
+	{
+	    plugin_set_current((Plugin *)dp);
             dp->init();
+	}
     }
 
 
     for (node = lowlevel_list; node; node = g_list_next(node)) {
         lp = LOWLEVEL_PLUGIN(node->data);
         if (lp->init)
+	{
+	    plugin_set_current((Plugin *)lp);
             lp->init();
+	}
     }
 
     if (cfg.disabled_iplugins) {
@@ -875,6 +999,7 @@ plugin_system_init(void)
 
         while (disabled[i]) {
             Plugin *plugintmp = plugin_get_plugin(disabled[i]);
+            g_free(disabled[i]);
             if (plugintmp)
                 INPUT_PLUGIN(plugintmp)->enabled = FALSE;
             i++;
@@ -904,6 +1029,7 @@ plugin_system_cleanup(void)
     LowlevelPlugin *lp;
     DiscoveryPlugin *dp;
     GList *node;
+    mowgli_node_t *hlist_node;
 
     g_message("Shutting down plugin system");
 
@@ -918,15 +1044,16 @@ plugin_system_cleanup(void)
 
     for (node = get_input_list(); node; node = g_list_next(node)) {
         ip = INPUT_PLUGIN(node->data);
-        if (ip && ip->cleanup) {
-            ip->cleanup();
+        if (ip) {
+	    plugin_set_current((Plugin *)ip);
+
+            if (ip->cleanup)
+                ip->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (ip->handle)
-            g_module_close(ip->handle);
     }
 
     if (ip_data.input_list != NULL)
@@ -937,17 +1064,18 @@ plugin_system_cleanup(void)
 
     for (node = get_output_list(); node; node = g_list_next(node)) {
         op = OUTPUT_PLUGIN(node->data);
-        if (op && op->cleanup) {
-            op->cleanup();
+        if (op) {
+	    plugin_set_current((Plugin *)op);
+
+            if (op->cleanup)
+                op->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (op->handle)
-            g_module_close(op->handle);
     }
-    
+
     if (op_data.output_list != NULL)
     {
         g_list_free(op_data.output_list);
@@ -956,15 +1084,16 @@ plugin_system_cleanup(void)
 
     for (node = get_effect_list(); node; node = g_list_next(node)) {
         ep = EFFECT_PLUGIN(node->data);
-        if (ep && ep->cleanup) {
-            ep->cleanup();
+        if (ep) {
+	    plugin_set_current((Plugin *)ep);
+
+            if (ep->cleanup)
+                ep->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (ep->handle)
-            g_module_close(ep->handle);
     }
 
     if (ep_data.effect_list != NULL)
@@ -975,15 +1104,16 @@ plugin_system_cleanup(void)
 
     for (node = get_general_list(); node; node = g_list_next(node)) {
         gp = GENERAL_PLUGIN(node->data);
-        if (gp && gp->cleanup) {
-            gp->cleanup();
+        if (gp) {
+	    plugin_set_current((Plugin *)gp);
+
+            if (gp->cleanup)
+                gp->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (gp->handle)
-            g_module_close(gp->handle);
     }
 
     if (gp_data.general_list != NULL)
@@ -994,15 +1124,16 @@ plugin_system_cleanup(void)
 
     for (node = get_vis_list(); node; node = g_list_next(node)) {
         vp = VIS_PLUGIN(node->data);
-        if (vp && vp->cleanup) {
-            vp->cleanup();
+        if (vp) {
+	    plugin_set_current((Plugin *)vp);
+
+            if (vp->cleanup)
+                vp->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (vp->handle)
-            g_module_close(vp->handle);
     }
 
     if (vp_data.vis_list != NULL)
@@ -1014,15 +1145,16 @@ plugin_system_cleanup(void)
 
     for (node = get_discovery_list(); node; node = g_list_next(node)) {
         dp = DISCOVERY_PLUGIN(node->data);
-        if (dp && dp->cleanup) {
-            dp->cleanup();
+        if (dp) {
+	    plugin_set_current((Plugin *)dp);
+
+            if (dp->cleanup)
+                dp->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (dp->handle)
-            g_module_close(dp->handle);
     }
 
     if (dp_data.discovery_list != NULL)
@@ -1031,19 +1163,18 @@ plugin_system_cleanup(void)
         dp_data.discovery_list = NULL;
     }
 
-
-
     for (node = lowlevel_list; node; node = g_list_next(node)) {
         lp = LOWLEVEL_PLUGIN(node->data);
-        if (lp && lp->cleanup) {
-            lp->cleanup();
+        if (lp) {
+	    plugin_set_current((Plugin *)lp);
+
+            if (lp->cleanup)
+                lp->cleanup();
+
             GDK_THREADS_LEAVE();
             while (g_main_context_iteration(NULL, FALSE));
             GDK_THREADS_ENTER();
         }
-
-        if (lp->handle)
-            g_module_close(lp->handle);
     }
 
     if (lowlevel_list != NULL)
@@ -1059,7 +1190,14 @@ plugin_system_cleanup(void)
         vfs_transports = NULL;
     }
 
+    MOWGLI_LIST_FOREACH(hlist_node, headers_list->head)
+    	plugin2_unload(hlist_node->data, hlist_node);
+
     mowgli_dictionary_destroy(plugin_dict, NULL, NULL);
+    mowgli_dictionary_destroy(pvt_data_dict, NULL, NULL);
+
+    mowgli_list_free(headers_list);
+
     g_hash_table_foreach(ext_hash, remove_list, NULL);
     g_hash_table_remove_all(ext_hash);
 }
