@@ -45,29 +45,23 @@ enum {
 
 static const gchar *audacious_brief =
     N_("<big><b>Audacious %s</b></big>\n"
-       "A skinned multimedia player for many platforms.\n"
+       "A multimedia player for many platforms.\n"
        "\n"
-       "Copyright (C) 2005-2008 Audacious Development Team\n");
+       "Copyright (C) 2005-2009 Audacious Development Team\n");
 
 static const gchar *credit_text[] = {
-    N_("Audacious core developers:"),
-    "George Averill",
-    "Daniel Barkalow",
+    N_("Core developers:"),
     "Christian Birchinger",
-    "Daniel Bradshaw",
-    "Adam Cecile",
     "Michael Färber",
-    "Troels Bang Jensen",
-    "Giacomo Lozito",
+    "Matti Hämäläinen",
+    "John Lindgren",
     "Cristi Măgherușan",
     "Tomasz Moń",
     "William Pitcock",
-    "Derek Pomery",
     "Jonathan Schleifer",
     "Ben Tucker",
     "Tony Vroon",
     "Yoshiki Yazawa",
-    "Eugene Zagidullin",
     NULL,
 
     N_("Graphics:"),
@@ -77,7 +71,7 @@ static const gchar *credit_text[] = {
 
     N_("Default skin:"),
     "George Averill",
-	"Michael Färber",
+    "Michael Färber",
     "William Pitcock",
     NULL,
 
@@ -91,38 +85,70 @@ static const gchar *credit_text[] = {
     "Sascha Hlusiak",
     "Giacomo Lozito",
     "Cristi Măgherușan",
+    "Boris Mikhaylov",
     "Tomasz Moń",
+    "Sebastian Pipping",
     "William Pitcock",
     "Derek Pomery",
     "Jonathan Schleifer",
+    "Andrew O. Shadoura",
     "Tony Vroon",
     "Yoshiki Yazawa",
     NULL,
 
     N_("Patch authors:"),
+    "Chris Arepantis",
+    "Carlo Bramini",
     "Massimo Cavalleri",
+    "Stefano D'Angelo",
+    "Jean-Louis Dupond",
     "Laszlo Dvornik",
     "Ralf Ertzinger",
     "Mike Frysinger",
     "Mark Glines",
+    "Hans de Goede",
+    "Jussi Judin",
     "Teru KAMOGASHIRA",
     "Chris Kehler",
+    "John Lindgren",
+    "Michał Lipski",
+    "Mark Loeser",
     "Alex Maclean",
     "Michael Hanselmann",
     "Joseph Jezak",
     "Henrik Johansson",
     "Rodrigo Martins de Matos Ventura",
     "Diego Pettenò",
+    "Edward Sheldrake",
     "Kazuki Shimura",
     "Valentine Sinitsyn",
     "Johan Tavelin",
+    "Bret Towe",
+    "John Wehle",
     "Tim Yamin",
+    "Ivan N. Zlatev",
     NULL,
 
-    N_("0.1.x developers:"),
+    N_("1.x developers:"),
+    "George Averill",
+    "Daniel Barkalow",
+    "Christian Birchinger",
+    "Daniel Bradshaw",
+    "Adam Cecile",
+    "Michael Färber",
+    "Matti Hämäläinen",
+    "Troels Bang Jensen",
+    "Giacomo Lozito",
+    "Cristi Măgherușan",
+    "Tomasz Moń",
     "William Pitcock",
+    "Derek Pomery",
     "Mohammed Sameer",
+    "Jonathan Schleifer",
+    "Ben Tucker",
     "Tony Vroon",
+    "Yoshiki Yazawa",
+    "Eugene Zagidullin",
     NULL,
 
     N_("BMP Developers:"),
@@ -157,6 +183,9 @@ static const gchar *credit_text[] = {
 };
 
 static const gchar *translators[] = {
+    N_("Basque:"),
+    "Iñaki Larrañaga Murgoitio",
+    NULL,
     N_("Brazilian Portuguese:"),
     "Fábio Antunes",
     "Philipi Pinto",
@@ -168,7 +197,7 @@ static const gchar *translators[] = {
     "Andrew Ivanov",
     NULL,
     N_("Catalan:"),
-    "Ernest Adrogu",
+    "Ernest Adrogué",
     NULL,
     N_("Croatian:"),
     "Marin Glibic",
@@ -185,6 +214,7 @@ static const gchar *translators[] = {
     NULL,
     N_("Finnish:"),
     "Pauli Virtanen",
+    "Matti Hämäläinen",
     NULL,
     N_("French:"),
     "Adam Cecile",
@@ -256,6 +286,8 @@ static const gchar *translators[] = {
     NULL,
     N_("Traditional Chinese:"),
     "Cheng-Wei Chien",
+    "Sylecn Song",
+    "Yang Zhang",
     NULL,
     N_("Turkish:"),
     "Murat Şenel",
@@ -414,7 +446,7 @@ show_credits_window(void)
 
     bbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
+    gtk_box_set_spacing(GTK_BOX(bbox), 5);
     gtk_box_pack_start(GTK_BOX(about_vbox), bbox, FALSE, FALSE, 0);
 
     close_btn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
