@@ -32,7 +32,6 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkmessagedialog.h>
 
 /* GDK including */
 #include "platform/smartinclude.h"
@@ -51,34 +50,21 @@
   #include <regex.h>
 #endif
 
-#include "ui_main.h"
 #include "icons-stock.h"
-
-#include "actions-mainwin.h"
-
 #include "main.h"
 
 #include "dnd.h"
-#include "dock.h"
-#include "hints.h"
 #include "input.h"
 #include "playback.h"
 #include "playlist.h"
 #include "pluginenum.h"
 #include "ui_credits.h"
-#include "ui_equalizer.h"
 #include "ui_fileopener.h"
-#include "ui_manager.h"
-#include "ui_playlist.h"
 #include "ui_preferences.h"
-#include "ui_skinselector.h"
-#include "ui_urlopener.h"
-#include "strings.h"
+#include "audstrings.h"
 #include "util.h"
 #include "visualization.h"
 
-#include "ui_skinned_window.h"
-
 #include "ui_jumptotrack_cache.h"
 
 static GtkWidget *jump_to_track_win = NULL;
@@ -278,9 +264,9 @@ ui_jump_to_track_update(GtkWidget * widget, gpointer user_data)
             gchar *realfn = NULL;
             realfn = g_filename_from_uri(entry->filename, NULL, NULL);
             if (strchr(realfn ? realfn : entry->filename, '/'))
-                desc_buf = str_to_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
+                desc_buf = str_assert_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
             else
-                desc_buf = str_to_utf8(realfn ? realfn : entry->filename);
+                desc_buf = str_assert_utf8(realfn ? realfn : entry->filename);
             g_free(realfn); realfn = NULL;
         }
 
@@ -345,9 +331,9 @@ ui_jump_to_track_edit_cb(GtkEntry * entry, gpointer user_data)
             gchar *realfn = NULL;
             realfn = g_filename_from_uri(entry->filename, NULL, NULL);
             if (strchr(realfn ? realfn : entry->filename, '/'))
-                title = str_to_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
+                title = str_assert_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
             else
-                title = str_to_utf8(realfn ? realfn : entry->filename);
+                title = str_assert_utf8(realfn ? realfn : entry->filename);
             g_free(realfn); realfn = NULL;
         }
         gtk_list_store_append(store, &iter);
@@ -398,9 +384,9 @@ ui_jump_to_track_fill(gpointer treeview)
             gchar *realfn = NULL;
             realfn = g_filename_from_uri(entry->filename, NULL, NULL);
             if (strchr(realfn ? realfn : entry->filename, '/'))
-                desc_buf = str_to_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
+                desc_buf = str_assert_utf8(strrchr(realfn ? realfn : entry->filename, '/') + 1);
             else
-                desc_buf = str_to_utf8(realfn ? realfn : entry->filename);
+                desc_buf = str_assert_utf8(realfn ? realfn : entry->filename);
             g_free(realfn); realfn = NULL;
         }
 
@@ -481,7 +467,7 @@ ui_jump_to_track(void)
     g_signal_connect(jump_to_track_win, "destroy",
                      G_CALLBACK(gtk_widget_destroyed), &jump_to_track_win);
 
-    gtk_container_border_width(GTK_CONTAINER(jump_to_track_win), 10);
+    gtk_container_set_border_width(GTK_CONTAINER(jump_to_track_win), 10);
     gtk_window_set_default_size(GTK_WINDOW(jump_to_track_win), 600, 500);
 
     vbox = gtk_vbox_new(FALSE, 5);
@@ -516,14 +502,14 @@ ui_jump_to_track(void)
     hbox = gtk_hbox_new(FALSE, 3);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);
 
-    
+
     /* filter box */
     search_label = gtk_label_new(_("Filter: "));
     gtk_label_set_markup_with_mnemonic(GTK_LABEL(search_label), _("_Filter:"));
     gtk_box_pack_start(GTK_BOX(hbox), search_label, FALSE, FALSE, 0);
 
     edit = gtk_entry_new();
-    gtk_entry_set_editable(GTK_ENTRY(edit), TRUE);
+    gtk_editable_set_editable(GTK_EDITABLE(edit), TRUE);
     gtk_label_set_mnemonic_widget(GTK_LABEL(search_label), edit);
     g_signal_connect(edit, "changed",
                      G_CALLBACK(ui_jump_to_track_edit_cb), treeview);
@@ -546,7 +532,9 @@ ui_jump_to_track(void)
                      toggle2);
 
     /* clear button */
-    rescan = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
+    rescan = gtk_button_new_with_mnemonic (_ ("Clea_r"));
+    gtk_button_set_image ((GtkButton *) rescan, gtk_image_new_from_stock
+     (GTK_STOCK_CLEAR, GTK_ICON_SIZE_BUTTON));
     gtk_box_pack_start(GTK_BOX(hbox), rescan, FALSE, FALSE, 0);
 
 
@@ -575,7 +563,7 @@ ui_jump_to_track(void)
 
     bbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 4);
+    gtk_box_set_spacing(GTK_BOX(bbox), 4);
     gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
 
     /* close dialog toggle */
@@ -583,16 +571,16 @@ ui_jump_to_track(void)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
                                  cfg.close_jtf_dialog ? TRUE : FALSE);
     gtk_box_pack_start(GTK_BOX(bbox), toggle, FALSE, FALSE, 0);
-    g_signal_connect(toggle, "clicked", 
+    g_signal_connect(toggle, "clicked",
                      G_CALLBACK(ui_jump_to_track_toggle_cb),
                      toggle);
-    
+
     queue = gtk_button_new_with_mnemonic(_("_Queue"));
     gtk_button_set_image(GTK_BUTTON(queue),
                      gtk_image_new_from_stock(AUD_STOCK_QUEUETOGGLE, GTK_ICON_SIZE_BUTTON));
     gtk_box_pack_start(GTK_BOX(bbox), queue, FALSE, FALSE, 0);
     GTK_WIDGET_SET_FLAGS(queue, GTK_CAN_DEFAULT);
-    g_signal_connect(queue, "clicked", 
+    g_signal_connect(queue, "clicked",
                      G_CALLBACK(ui_jump_to_track_queue_cb),
                      treeview);
     g_signal_connect(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), "changed",
